@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -34,42 +34,47 @@
 #include	"md_u.h"
 #include	"md_p.h"
 
-int Kill(char *dev, int force)
+int Kill(char *dev, int force, int quiet)
 {
 	/*
 	 * Nothing fancy about Kill.  It just zeroes out a superblock
 	 * Definitely not safe.
 	 */
 
-	void *super;
 	int fd, rv = 0;
 	struct supertype *st;
-		
-	fd = open(dev, O_RDWR|O_EXCL);
+
+	fd = open(dev, O_RDWR|(force ? 0 : O_EXCL));
 	if (fd < 0) {
-		fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
-			dev);
+		if (!quiet)
+			fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
+				dev);
+		close(fd);
 		return 1;
 	}
 	st = guess_super(fd);
 	if (st == NULL) {
-		fprintf(stderr, Name ": Unrecognised md component device - %s\n", dev);
+		if (!quiet)
+			fprintf(stderr, Name ": Unrecognised md component device - %s\n", dev);
+		close(fd);
 		return 1;
 	}
-	rv = st->ss->load_super(st, fd, &super, dev);
+	rv = st->ss->load_super(st, fd, dev);
 	if (force && rv >= 2)
 		rv = 0; /* ignore bad data in superblock */
 	if (rv== 0 || (force && rv >= 2)) {
 		mdu_array_info_t info;
 		info.major_version = -1; /* zero superblock */
-		free(super);
-		st->ss->init_super(st, &super, &info, "");
-		if (st->ss->store_super(st, fd, super)) {
-			fprintf(stderr, Name ": Could not zero superblock on %s\n",
-				dev);
+		st->ss->free_super(st);
+		st->ss->init_super(st, &info, 0, "", NULL, NULL);
+		if (st->ss->store_super(st, fd)) {
+			if (!quiet)
+				fprintf(stderr, Name ": Could not zero superblock on %s\n",
+					dev);
 			rv = 1;
 		} else if (rv) {
-			fprintf(stderr, Name ": superblock zeroed anyway\n");
+			if (!quiet)
+				fprintf(stderr, Name ": superblock zeroed anyway\n");
 			rv = 0;
 		}
 	}
