@@ -29,7 +29,7 @@
 
 #include	"mdadm.h"
 #include	"dlink.h"
-#include	<sys/dir.h>
+#include	<dirent.h>
 #include	<glob.h>
 #include	<fnmatch.h>
 #include	<ctype.h>
@@ -86,7 +86,7 @@ char *keywords[] = {
 	[Mailaddr] = "mailaddr",
 	[Mailfrom] = "mailfrom",
 	[Program]  = "program",
-	[CreateDev]   = "create",
+	[CreateDev]= "create",
 	[Homehost] = "homehost",
 	[LTEnd]    = NULL
 };
@@ -100,7 +100,7 @@ int match_keyword(char *word)
 {
 	int len = strlen(word);
 	int n;
-    
+
 	if (len < 3) return -1;
 	for (n=0; keywords[n]; n++) {
 		if (strncasecmp(word, keywords[n], len)==0)
@@ -161,11 +161,24 @@ char *conf_word(FILE *file, int allow_key)
 					word[len++] = c;
 				}
 				c = getc(file);
+				/* Hack for broken kernels (2.6.14-.24) that put
+				 *        "active(auto-read-only)"
+				 * in /proc/mdstat instead of
+				 *        "active (auto-read-only)"
+				 */
+				if (c == '(' && len >= 6
+				    && strncmp(word+len-6, "active", 6) == 0)
+					c = ' ';
 			}
 		}
 		if (c != EOF) ungetc(c, file);
 	}
 	word[len] = 0;
+
+	/* Further HACK for broken kernels.. 2.6.14-2.6.24 */
+	if (strcmp(word, "auto-read-only)") == 0)
+		strcpy(word, "(auto-read-only)");
+
 /*    printf("word is <%s>\n", word); */
 	if (!wordfound) {
 		free(word);
@@ -173,7 +186,7 @@ char *conf_word(FILE *file, int allow_key)
 	}
 	return word;
 }
-	
+
 /*
  * conf_line reads one logical line from the conffile.
  * It skips comments and continues until it finds a line that starts
@@ -237,12 +250,13 @@ mddev_dev_t load_partitions(void)
 		if (buf[0] != ' ')
 			continue;
 		major = strtoul(buf, &mp, 10);
-		if (mp == buf || *mp != ' ') 
+		if (mp == buf || *mp != ' ')
 			continue;
 		minor = strtoul(mp, NULL, 10);
 
 		name = map_dev(major, minor, 1);
-
+		if (!name)
+			continue;
 		d = malloc(sizeof(*d));
 		d->devname = strdup(name);
 		d->next = rv;
@@ -254,6 +268,8 @@ mddev_dev_t load_partitions(void)
 }
 
 struct createinfo createinfo = {
+	.autof = 2, /* by default, create devices with standard names */
+	.symlinks = 1,
 #ifdef DEBIAN
 	.gid = 6, /* disk */
 	.mode = 0660,
@@ -289,6 +305,8 @@ int parse_auto(char *str, char *msg, int config)
 		len = e - str;
 		if ((len == 2 && strncasecmp(str,"md",2)==0)) {
 			autof = config ? 5 : 3;
+		} else if ((len == 3 && strncasecmp(str,"yes",3)==0)) {
+			autof = 2;
 		} else if ((len == 3 && strncasecmp(str,"mdp",3)==0)) {
 			autof = config ? 6 : 4;
 		} else if ((len == 1 && strncasecmp(str,"p",1)==0) ||
@@ -320,14 +338,12 @@ static void createline(char *line)
 			}
 			createinfo.uid = strtoul(w+6, &ep, 10);
 			if (*ep != 0) {
-#ifndef STATIC
 				struct passwd *pw;
 				/* must be a name */
 				pw = getpwnam(w+6);
 				if (pw)
 					createinfo.uid = pw->pw_uid;
 				else
-#endif /* STATIC */
 					fprintf(stderr, Name ": CREATE user %s not found\n", w+6);
 			}
 		} else if (strncasecmp(w, "group=", 6) == 0) {
@@ -337,14 +353,12 @@ static void createline(char *line)
 			}
 			createinfo.gid = strtoul(w+6, &ep, 10);
 			if (*ep != 0) {
-#ifndef STATIC
 				struct group *gr;
 				/* must be a name */
 				gr = getgrnam(w+6);
 				if (gr)
 					createinfo.gid = gr->gr_gid;
 				else
-#endif /* STATIC */
 					fprintf(stderr, Name ": CREATE group %s not found\n", w+6);
 			}
 		} else if (strncasecmp(w, "mode=", 5) == 0) {
@@ -358,14 +372,27 @@ static void createline(char *line)
 				fprintf(stderr, Name ": unrecognised CREATE mode %s\n",
 					w+5);
 			}
-		} else {
+		} else if (strncasecmp(w, "metadata=", 9) == 0) {
+			/* style of metadata to use by default */
+			int i;
+			for (i=0; superlist[i] && !createinfo.supertype; i++)
+				createinfo.supertype =
+					superlist[i]->match_metadata_desc(w+9);
+			if (!createinfo.supertype)
+				fprintf(stderr, Name ": metadata format %s unknown, ignoring\n",
+					w+9);
+		} else if (strncasecmp(w, "symlinks=yes", 12) == 0)
+			createinfo.symlinks = 1;
+		else if  (strncasecmp(w, "symlinks=no", 11) == 0)
+			createinfo.symlinks = 0;
+		else {
 			fprintf(stderr, Name ": unrecognised word on CREATE line: %s\n",
 				w);
 		}
 	}
 }
 
-void devline(char *line) 
+void devline(char *line)
 {
 	char *w;
 	struct conf_dev *cd;
@@ -451,7 +478,7 @@ void arrayline(char *line)
 				fprintf(stderr, Name ": only specify bitmap file once. %s ignored\n",
 					w);
 			else
-				mis.bitmap_file = w+7;
+				mis.bitmap_file = strdup(w+7);
 
 		} else if (strncasecmp(w, "devices=", 8 ) == 0 ) {
 			if (mis.devices)
@@ -480,7 +507,7 @@ void arrayline(char *line)
 		} else if (strncasecmp(w, "metadata=", 9) == 0) {
 			/* style of metadata on the devices. */
 			int i;
-			
+
 			for(i=0; superlist[i] && !mis.st; i++)
 				mis.st = superlist[i]->match_metadata_desc(w+9);
 
@@ -532,7 +559,7 @@ void mailfromline(char *line)
 			alert_mail_from = strdup(w);
 		else {
 			char *t= NULL;
-			asprintf(&t, "%s %s", alert_mail_from, w);
+			xasprintf(&t, "%s %s", alert_mail_from, w);
 			free(alert_mail_from);
 			alert_mail_from = t;
 		}
@@ -571,7 +598,13 @@ void homehostline(char *line)
 
 int loaded = 0;
 
-void load_conffile(char *conffile)
+static char *conffile = NULL;
+void set_conffile(char *file)
+{
+	conffile = file;
+}
+
+void load_conffile(void)
 {
 	FILE *f;
 	char *line;
@@ -637,53 +670,53 @@ void load_conffile(char *conffile)
 		}
 		free_line(line);
 	}
-    
+
 	fclose(f);
 
 /*    printf("got file\n"); */
 }
 
-char *conf_get_mailaddr(char *conffile)
+char *conf_get_mailaddr(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return alert_email;
 }
 
-char *conf_get_mailfrom(char *conffile)
+char *conf_get_mailfrom(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return alert_mail_from;
 }
 
-char *conf_get_program(char *conffile)
+char *conf_get_program(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return alert_program;
 }
 
-char *conf_get_homehost(char *conffile)
+char *conf_get_homehost(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return home_host;
 }
 
-struct createinfo *conf_get_create_info(char *conffile)
+struct createinfo *conf_get_create_info(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return &createinfo;
 }
 
-mddev_ident_t conf_get_ident(char *conffile, char *dev)
+mddev_ident_t conf_get_ident(char *dev)
 {
 	mddev_ident_t rv;
-	load_conffile(conffile);
+	load_conffile();
 	rv = mddevlist;
 	while (dev && rv && strcmp(dev, rv->devname)!=0)
 		rv = rv->next;
 	return rv;
 }
 
-mddev_dev_t conf_get_devs(char *conffile)
+mddev_dev_t conf_get_devs()
 {
 	glob_t globbuf;
 	struct conf_dev *cd;
@@ -697,8 +730,8 @@ mddev_dev_t conf_get_devs(char *conffile)
 		free(t->devname);
 		free(t);
 	}
-    
-	load_conffile(conffile);
+
+	load_conffile();
 
 	if (cdevlist == NULL)
 		/* default to 'partitions */
@@ -727,6 +760,22 @@ mddev_dev_t conf_get_devs(char *conffile)
 	return dlist;
 }
 
+int conf_test_dev(char *devname)
+{
+	struct conf_dev *cd;
+	if (cdevlist == NULL)
+		/* allow anything by default */
+		return 1;
+	for (cd = cdevlist ; cd ; cd = cd->next) {
+		if (strcasecmp(cd->name, "partitions") == 0)
+			return 1;
+		if (fnmatch(cd->name, devname, FNM_PATHNAME) == 0)
+			return 1;
+	}
+	return 0;
+}
+
+
 int match_oneof(char *devices, char *devname)
 {
     /* check if one of the comma separated patterns in devices
